 # -*- coding: UTF-8 -*-
# Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import time
import select
import socket
import threading

from scapy.data import ETH_P_ALL
from scapy.arch.linux import _flush_fd
from sndrcvth import attach_filter

class CancelMonitor:
    
    def __init__(self):
        self.state=False
        self.lock=threading.Lock()

    def is_cancelled(self):
        tmp=None
        self.lock.acquire()
        tmp=self.state
        self.lock.release()
        return tmp

    def set_cancelled(self):
        self.lock.acquire()
        self.state=True
        self.lock.release()

    def clear_cancelled(self):
        self.lock.acquire()
        self.state=False
        self.lock.release()


class PacketFinder(object):
   
    MAXBYTES=2500

    def __init__(self, srvconf, socketfilter, packetfilter, _timeout):

        self.filter=packetfilter
        self.timeout=_timeout
        # @type srvconf ServerConf
        self.llpkt=srvconf.get_monitor_packet()
        self.sock=socket.socket(socket.AF_PACKET,
                                socket.SOCK_RAW,
                                socket.htons(ETH_P_ALL))

        
        self.sock.bind((srvconf.get_if_mon(), ETH_P_ALL))
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 2 ** 20)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2**8)
        attach_filter(self.sock, srvconf.get_if_mon(), socketfilter)        
        _flush_fd(self.sock)

    def find(self):
        
        input=[self.sock]
        remaintime=float(self.timeout)
        found=False
        fdtuple=None
        buf=None        
        pkt=None
        elapsed=0
        checktime=0

        while(remaintime>0 and not found):
            checktime=time.time()
            fdtuple=select.select(input, [], [], remaintime)

            if(self.sock in fdtuple[0]):
                buf, _ =self.sock.recvfrom(self.MAXBYTES)
                pkt=self.llpkt(buf)
                
                for filter in self.filter:
                    if(filter(pkt)):
                        found=True
                        break
                        
                if(found):
                    pass
                else:
                    elapsed=time.time()-checktime
                    remaintime=remaintime-elapsed
            else:
                remaintime=0

               
        if found:
            return pkt
        else:
            return None        
        

    def release(self):
        self.sock.close()
        

class IPSuffixGen(object):
       
    def next(self):
        pass

    def __iter__(self):
        return self


class IPSGenAscComp(IPSuffixGen):    

    def __init__(self):
        self.fst=0
        self.snd=0

    def next(self):

        if self.snd>=254:
            
            if self.fst>=255:
                raise StopIteration
            else:
                self.snd=1
                self.fst=self.fst+1

        else:
            
            self.snd=self.snd+1
            

        return (self.fst, self.snd)



class IPSGenPreParc(IPSuffixGen):
    

    def __init__(self):
        self.sndrange=[1, 254, 100, 200, 50, 150, 250]
        self.fst=-1
        self.snd=0

    def next(self):

        if self.fst>254:

            if self.snd<len(self.sndrange)-1:
                self.fst=0
                self.snd=self.snd+1
            else:
                raise StopIteration
                
        else:

            self.fst=self.fst+1


        return (self.fst, self.sndrange[self.snd])

